package VA;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Scanner;
import Auxiliar.Direccao;
import Avaliador.AvaliadorAmbiente;
import Robot.RobotHemissonActivo;
import Tarefa.DetectarRobot;
import Tarefa.PassearFugir;
import Tarefa.PassearFugirAgressivo;
import Tarefa.PassearFugirAgressivoTurnos;
import Tarefa.PassearFugirDetect;
import Tarefa.SegueLinha;
import Tarefa.TarefaRobot;

public class Controle {

	private String[] arguments;
	private RobotHemissonActivo robot;
	
	private static final int esquerda = 5;
	private static final int direita = 3;
	
	private Collection <RobotHemissonActivo> myRobots;
	
	public Controle(String[] args) {
		this.arguments = args;
		processArguments();
		generateApplicationHeader();
		myRobots = new ArrayList <RobotHemissonActivo> ();
	}
	
	private void generateApplicationHeader() {
		System.out.println("Hemisson Console (version 1.0)");
		System.out.println("Copyright: LabMag (c) 2006");
		System.out.println();
	}

	private void processArguments() {
		if(this.arguments.length > 0) {
			System.out.print("Argumentos: " + arguments[0]);
			for(int i = 1; i<this.arguments.length; i++)
				System.out.print(", " + arguments[i]);
			System.out.println(": Foram todos ignorados :D [no temos argumentos definidos ainda]");
		}
	}

	private boolean setRobot(String name) {
		boolean status = true;
		int index = findRobot(name);
		if(index > -1) {
			status = setRobot(index);
		} else {
			status = false;
			System.out.println("No existe nenhum robot com o nome: " + name);
		}
		return status;
	}
	
	private boolean setRobot (int index) {
		boolean status = true;
		RobotHemissonActivo temp = null;
		if(index >= myRobots.size()) {
			status = false;
			System.out.println("Identificador de robot invlido");
		}
		if(status) {
			temp = (RobotHemissonActivo) myRobots.toArray()[index];
			if(temp == null) {
				status = false;
				System.out.println("O Robot pretendido no existe");
			} else {
				robot = temp;
			}
		}
		
		return status;
	}
	
	private boolean createRobot(String name, int commPort) {
		boolean status = true;
		RobotHemissonActivo temp = new RobotHemissonActivo("COM"+commPort,name);
		if(temp != null) {
			myRobots.add(temp);
			robot = temp; //tornar este o robot corrente
		} else {
			System.out.println("Ocorreu um erro ao inicializar o Robot");
			status = false;
		}
		
		return status;
	}
	
	private boolean disconnectRobot(String name) {
		boolean status = true;
		int index = findRobot(name);
		if(index > -1) {
			status = disconnectRobot(index);
		} else {
			status = false;
			System.out.println("No existe nenhum robot com o nome: " + name);
		}
		return status;
	}
	
	private boolean disconnectRobot(int index) {
		boolean status = true;
		RobotHemissonActivo temp = null;
		if(index >= myRobots.size()) {
			status = false;
			System.out.println("Identificador de robot invlido");
		}
		if(status) {
			temp = (RobotHemissonActivo) myRobots.toArray()[index];
			if(temp == null) {
				status = false;
				System.out.println("O Robot pretendido no existe");
			} else {
				status = myRobots.remove(temp);
				if(!status)
					System.out.println("Ocorreu um erro ao remover o robot da lista de robots");
			}
		}
		
		return status;
	}
	
	private int findRobot (String name) {
		RobotHemissonActivo lista[] = (RobotHemissonActivo[]) myRobots.toArray();
		boolean found = false;
		int position = 0;
		for(int i = 0; !found && i<lista.length; i++)
			if(lista[i].getNome().equalsIgnoreCase(name)) {
				found = true;
				position = i;
			}
		
		if(found)
			return position;
		else 
			return -1;
	}
	
	private void testAllRobot() {
		System.out.println("A emitir sons com todos os robos...");
		RobotHemissonActivo lista[] = (RobotHemissonActivo[]) myRobots.toArray();
		for(int i = 0; i<lista.length; i++) {
				System.out.println("Robot " + i + ": A emitir " + (i + 1) + " beeps");
				lista[i].beeper(i+1);
		}
	}
	
	public void run() throws Exception {
		Scanner sc = new Scanner(System.in);
		String command;
		while(true) {
			try {
				System.out.print("Cmd: ");
				command = sc.next();
				if(command.equalsIgnoreCase("help")) { //Printa informao de help
					printHelp();
				} else if(command.equalsIgnoreCase("infmove")) { //Executa um movimento infinito
					makeMove(sc,false);
				} else if(command.equalsIgnoreCase("move")) { //Executa um movimento
					makeMove(sc,true);
				} else if(command.equalsIgnoreCase("read")) { //Executa uma leitura dos sensores de sonar
					makeRead();
				} else if(command.equalsIgnoreCase("readl")) { //Executa uma leitura dos sensores de luz
					makeReadLight();
				} else if(command.equalsIgnoreCase("rt")) { //Executa um movimento de rotao orientado
					makeRotation(sc);
				} else if(command.equalsIgnoreCase("stop")) { //Executa uma paragem completa do robot
					robot.setSpeed(0,0);
				} else if(command.equalsIgnoreCase("teste")) { //Executar um teste em todos os robos activos
					testAllRobot();
				} else if(command.equalsIgnoreCase("execute")) { //Executa uma tarefa
					executeTarefa(sc);
				} else if(command.equalsIgnoreCase("terminar")) { //TErminar a tarefa do robot corrent;
					try {
						robot.terminarTarefa();
						System.out.println("Tarefa terminada.");
					} catch (Exception ex) {
						System.out.println("Tarefa no foi terminada: " + ex.getMessage());
					}
				} else if(command.equalsIgnoreCase("quit")) {
					break;
				} else if(command.equalsIgnoreCase("create")) {
					this.createRobot(sc.next(),sc.nextInt());
				} else if(command.equalsIgnoreCase("remove")) {
					removeRobot(sc);
				} else if(command.equalsIgnoreCase("activate")){
					activateRobot(sc);
				} else {
					System.out.println("Comando Desconhecido");
				}
			
				if(robot != null)
					robot.beeper(1); //Para se saber que o robot recebeu um comando
			} catch (Exception e) {
				System.out.println("Erro: " + e.getMessage());
			}
		}
	}
	
	private void activateRobot(Scanner sc) {
		String target = sc.next();
		boolean status = false;
		try {
			status = this.setRobot(Integer.parseInt(target));
		} catch (Exception e) {
			status = this.setRobot(target);
		}
		
		if(status) {
			System.out.println("robot " + target + "  o novo robot activo");
		}
	}

	private void removeRobot(Scanner sc) {
		String target = sc.next();
		boolean status = false;
		try {
			status = this.disconnectRobot(Integer.parseInt(target));
		} catch (Exception e) {
			status = this.disconnectRobot(target);
		}
		
		if(status) {
			System.out.println("robot removido");
		}
	}

	private void executeTarefa(Scanner sc) {
		String command = sc.next();
		
		TarefaRobot th = null;
		
		if(command.equalsIgnoreCase("passear")) 
			th = new PassearFugir(robot,null);
		else if(command.equalsIgnoreCase("linha")) {
			AvaliadorAmbiente a = new AvaliadorAmbiente(robot); //  necessrio um avaliador
			a.init(); //incializar o avaliador
			th = new SegueLinha(robot,a);
		} else if(command.equals("passearFast"))
			th = new PassearFugirAgressivo(robot,null);
		else if(command.equals("passearTurnos"))
			th = new PassearFugirAgressivoTurnos(robot,null);
		else if(command.equals("passearDetect"))
			th = new PassearFugirDetect(robot,null);
		else if(command.equals("detect"))
			th = new DetectarRobot(robot,null);
		if(th != null) {
			robot.executaTarefa(th);
			System.out.println("Tarefa incializada...");
		} else
			System.out.println("A tarefa pretendida  desconhecida");
	}

	private void makeRotation(Scanner sc) throws Exception {
		String command = sc.next();
		Direccao d = null;
		if(command.equalsIgnoreCase("direita"))
			d = new Direccao(Direccao.direita);
		else if(command.equalsIgnoreCase("esquerda"))
			d = new Direccao(Direccao.esquerda);
		else 
			d = new Direccao(Direccao.tras);
		
		d.executarMovimento(robot);	
	}

	private void printHelp() {
		System.out.println("Consola para controle de Robots Hemisson");
		System.out.println();
		System.out.println("Comandos genricos da consola");
		System.out.println("Help: \tMostra esta mensagem");
		System.out.println("Teste: \tExecuta um teste para verificar ligao com robots");
		System.out.println("Quit: \tTermina a consola");
		System.out.println();
		System.out.println("Comandos disponveis para manipular robot:");
		System.out.println("Create Nome CommPort: \tCria a ligao a um novo robot, o robot ter o nome e deve estar ligado na port indicada (valor numrico apenas)");
		System.out.println("Remove Robot: \t\tPerde os dados de um robot e termina as suas tarefas, o identificador pode ser o nome ou index do robot");
		System.out.println("Activate Robot: \tColoca o robot indicado (nome ou id) como sendo o robot activo");
		System.out.println();		
		System.out.println("Comandos disponveis para manipular o robot activo:");
		System.out.println("infmove Esq Dir: \tEfectua o movimento com os valores Esq e Dir nos motores esquerdo e direito, e desactiva a consola");
		System.out.println("move Esq Dir: \t\tEfectua o movimento com os valores Esq e Dir nos motores durante 1 segundo");
		System.out.println("stop: \t\t\tcoloca valores zero em ambos os motores do robot");
		System.out.println("read: \t\t\tMostra os valores obtidos pelos sensores de distncia (Sonar)");
		System.out.println("readl: \t\t\tMostra os valores obtidos pelos sensores de luz (Light)");
		System.out.println("rt Direccao: \t\tPermite efectuar um movimento de rotao");
		System.out.println("\t Direccao pode tomar valores: esquerda, direita, tras");
		System.out.println();		
		System.out.println("Comandos disponveis para manipular tarefas no robot activo");
		System.out.println("execute Tarefa: \tLana uma thread que executa uma certa operao");
		System.out.println("\t Tarefa pode tomar os seguintes valores:");
		System.out.println("\t passear - Tarefa para passear num mundo desviando-se de obstculos");
		System.out.println("\t linha - Tarefa para seguir uma linha preta num cho (branco); envolve calibrao de sensores");
		System.out.println("\t detect - Tarefa para passear e detectar outro robot (objecto em movimento) num espao");
		System.out.println("\t passearFast - Tarefa para passear num mundo desviando-se de obstculos (algoritmo mais agressivo)");
		System.out.println("\t passearTurnos - Tarefa para passear num mundo desviando-se de obstculos (algoritmo mais agressivo com pausas)");
		System.out.println("\t passearDetect - Tarefa para passear num mundo desviando-se de obstculos e parar quando detecta um robot");
		System.out.println();
	}
	
	private void makeReadLight() {
		int[] values = robot.readLight();
		for(int i = 0; i<values.length; i++) {
			System.out.print(values[i] + ";");	
		}
		System.out.println("Chao: " + values[esquerda] + " : " + values[direita] );
	}

	private void makeRead() {
		int[] values = robot.readSonar();
		for(int i = 0; i<values.length; i++) {
			System.out.print(values[i] + ";");	
		}
		System.out.println("Chao: " + values[esquerda] + " : " + values[direita] );
	}

	private void makeMove(Scanner sc, boolean stop) throws InterruptedException {
		int esquerda = sc.nextInt();
		int direita = sc.nextInt();
		robot.setSpeed(esquerda,direita);
		if(stop) {
			Thread.sleep(1000);
			robot.setSpeed(0,0);
		} else {
			System.exit(0);
		}
	}

	
	/**
	 * @param args
	 * @throws InterruptedException 
	 */
	public static void main(String[] args) throws InterruptedException {

		Controle c = new Controle(args);
		try{
			c.run();
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}

	}

}
