package hemissonComm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Enumeration;
import javax.comm.CommPortIdentifier;
import javax.comm.PortInUseException;
import javax.comm.SerialPort;
import javax.comm.UnsupportedCommOperationException;
import com.sun.comm.Win32Driver;

/**
 * Classe que implementa os comandos para os robots Hemisson
 * 
 * @author hal
 *
 */

public abstract class CommandLibrary{
	
	private  Enumeration portList;
	private  CommPortIdentifier portId;
	private  SerialPort serialPort;
	private  OutputStream outputStream;
	private  BufferedReader sensorReader;
	
	/**
	 * Construtor de uma ligao a um robot
	 * @param portName nome da porta (COM4 a COM7)
	 */
	/*		public CommandLibrary(String portName) {
	 super(portName);
	 }
	 */		
	public CommandLibrary(String portName) {
		super();
		//No sei para que  isto...
		//Win32Driver w32Driver = new Win32Driver();
		//w32Driver.initialize();
		portList = CommPortIdentifier.getPortIdentifiers();
		
		while (portList.hasMoreElements()) {
			portId = (CommPortIdentifier) portList.nextElement();
			if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) 
				if (portId.getName().equals(portName)) {
					try {
						serialPort = (SerialPort)
						portId.open("HemissonCom", 30000);
					} catch (PortInUseException e) 
					{
						System.out.println("Erro: Porta " + portName + "a ser utilizada");
						e.printStackTrace();
					}
					try {
						outputStream = serialPort.getOutputStream();
					} catch (IOException e) 
					{
						System.out.println("Erro: No foi possivel abrir um OutputStream sobre "+ portName);
						e.printStackTrace();
					}
					try {
						serialPort.setSerialPortParams(115200,
								SerialPort.DATABITS_8,
								SerialPort.STOPBITS_1,
								SerialPort.PARITY_NONE);
						sensorReader = new BufferedReader(new InputStreamReader(serialPort.getInputStream()));
					} catch (UnsupportedCommOperationException e) 
					{
						System.out.println("Erro: Operao no suportada");
						e.printStackTrace(); 
					}
					catch (IOException e) {
						System.out.println("Erro: No foi possivel abrir um InputStream sobre "+portName);
						e.printStackTrace();
					}
				}
		}
	}
	
	/**
	 * Coloca o robot em movimento com um dada velocidade de translaco
	 * e uma dada velocidade de rotao. Este metodo apenas existe 
	 * para efeitos de compatibilidade com o Player/Stage
	 * @param fw real representando a velocidade de translaco
	 * @param tw real representando a velociadade de rotao
	 */
	public void setSpeedPS(double fw, double tw)
	{
		int lm=(int)fw,rm=(int)fw;
		if (fw>0)
			if (tw < 0) lm +=(int)tw; else rm-=(int)tw;
		else
			if (tw < 0) lm -=(int)tw; else rm+=(int)tw;
		lm = Math.max(Math.min(9,lm),-9);
		rm = Math.max(Math.min(9,rm),-9);
		//sendCommand("D,"+lm+","+rm);
		try {
			outputStream.write(("D,"+lm+","+rm+"\r").getBytes());
			sensorReader.readLine(); //Temos de limpar o resultado da operao
		} catch (IOException e) {e.printStackTrace();} 
	}
	
	/**
	 * Coloca o robot em movimento imprimindo uma dada velocidade na roda esquerda
	 * e uma dada velocidade na roda esquerda. 
	 * @param lm inteiro representando a velocidade esquerda
	 * @param rm inteiro representando a velociadade direita
	 */
	public void setSpeed(int lm, int rm)
	{
		
		lm = Math.max(Math.min(9,lm),-9);
		rm = Math.max(Math.min(9,rm),-9);
		//sendCommand("D,"+lm+","+rm);			
		try {
			outputStream.write(("D,"+lm+","+rm+"\r").getBytes());
			sensorReader.readLine();
		} catch (IOException e) {e.printStackTrace();} 
	}
	
	/**
	 * Devolve o valor relativo da velocidade do robot. S para efeitos 
	 * de compatibilidade com o Player/Stage. 
	 * @return uma tabela com os valores da velocidade de translao e rotao
	 */
	public int[] readSpeedPS()
	{
		int[] aux = new int[2];
		int lm=0,rm=0;
		String[] s;
		
		try {
			outputStream.write(("E\r").getBytes());
			s=sensorReader.readLine().split(",");
			
			//s=sendCommand("E").readLine().split(",");
			lm = new Integer(s[1]);
			rm = new Integer(s[2]);
		} catch (IOException e) {e.printStackTrace();} 
		
		
		aux[0]=lm+rm/2;
		aux[1]=lm-rm;
		
		return aux;
	}
	
	
	/**
	 * Devolve o valor da potncia aplicada a cada motor do robot. 
	 * @return uma tabela com os valores da esquerda e direita.
	 */
	public int[] readSpeed()
	{
		int[] aux = new int[2];
		String[] s;
		
		try {
			outputStream.write(("E\r").getBytes());
			s=sensorReader.readLine().split(",");
			
			//s=sendCommand("E").readLine().split(",");
			aux[0] = new Integer(s[1]);
			aux[1] = new Integer(s[2]);
		} catch (IOException e) {e.printStackTrace();} 
		return aux;
	}
	
	/**
	 * Devolve os valores dos sensores de proximidade segundo a seguinte ordem
	 * Front,FrontRight,Right,GroundRight,Rear,GroundLeft,Left,FrontLeft
	 * @return tabela de 8 inteiros entre 0 e 255 onde 255 representa o maximo de 
	 * proximidade
	 */
	public int[] readSonar()
	{
		return readData("N");
	}
	
	/**
	 * Devolve os valores dos sensores de luminosidade segundo a seguinte ordem
	 * Front,FrontRight,Right,GroundRight,Rear,GroundLeft,Left,FrontLeft
	 * @return tabela de 8 inteiros entre 0 e 255 onde 255 representa a ausncia 
	 * de luminosidade
	 */
	public int[] readLight()
	{
		return readData("O");
	}
	
	private int[] readData(String c)
	{
		int[] aux1 = new int[8],aux = new int[8];
		String[] s;
		
		try {
			outputStream.write((c+"\r").getBytes());
			s=sensorReader.readLine().split(",");
			
			//s=sendCommand("c").readLine().split(",");
			for(int i=0; i<8 ; i++)
				aux1[i] = new Integer(s[i+1]);
		} catch (IOException e) {e.printStackTrace();} 
		
		
		aux[0]= aux1[0];	//Front
		aux[1]= aux1[1];	//FrontRigth
		aux[2]= aux1[3];	//Rigth
		aux[3]= aux1[6];	//GroundRigth
		aux[4]= aux1[5];	//Rear
		aux[5]= aux1[7];	//GroundLeft
		aux[6]= aux1[4];	//Left
		aux[7]= aux1[2];	//FrontLeft
		
		return aux;
		
	}
	
	/**
	 * Emite uma sequncia de n beeps com durao de 50ms e 50ms de intervalo
	 * @param times numero de beeps a serem emitidos
	 */
	public void beeper(int times) 
	{
		for(int i=0;i<times;i++)
		{
			try {
				outputStream.write(("H,1\r").getBytes());
				sensorReader.readLine();
				Thread.sleep(50);
				outputStream.write(("H,0\r").getBytes());
				sensorReader.readLine();
				Thread.sleep(50);	           		
			} catch (IOException e) {e.printStackTrace();} 
			catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 
			
		}
	}
	
	/**
	 * Fecho de todos os ports e streams abertos
	 */
	public void close() {
		try {
			sensorReader.close();
			outputStream.close();
		} catch (IOException e) {
		}
		serialPort.close();
	}
	
}
